@@ -7,6 +7,7 @@ from __future__ import unicode_literals
 
 import contextlib
 import io
+import os
 import subprocess
 import sys
 import tempfile
@@ -15,6 +16,25 @@ import unittest
 import autoflake
 
 
+ROOT_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
+
+
+if (
+    'AUTOFLAKE_COVERAGE' in os.environ and
+    int(os.environ['AUTOFLAKE_COVERAGE'])
+):
+    AUTOFLAKE_COMMAND = ['coverage', 'run', '--branch', '--parallel',
+                         '--omit=*/distutils/*,*/site-packages/*',
+                         os.path.join(ROOT_DIRECTORY, 'autoflake.py')]
+else:
+    # We need to specify the executable to make sure the correct Python
+    # interpreter gets used.
+    AUTOFLAKE_COMMAND = [sys.executable,
+                         os.path.join(
+                             ROOT_DIRECTORY,
+                             'autoflake.py')]  # pragma: no cover
+
+
 class UnitTests(unittest.TestCase):
 
     """Unit tests."""
@@ -411,6 +431,30 @@ def bar():
             self.assertEqual('latin-1',
                              autoflake.detect_encoding(filename))
 
+    def test_fix_code_with_comma_on_right(self):
+        """pyflakes does not handle nonlocal correctly."""
+        self.assertEqual(
+            """\
+def main():
+    pass
+""",
+            autoflake.fix_code("""\
+def main():
+    x = (1, 2, 3)
+""",
+                               remove_unused_variables=True))
+
+    def test_fix_code_with_unused_variables_should_skip_multiple(self):
+        code = """\
+def main():
+    (x, y, z) = (1, 2, 3)
+    print(z)
+"""
+        self.assertEqual(
+            code,
+            autoflake.fix_code(code,
+                               remove_unused_variables=True))
+
     def test_useless_pass_line_numbers(self):
         self.assertEqual(
             [1],
@@ -602,6 +646,8 @@ def func11():
     def test_check_with_unicode(self):
         self.assertFalse(autoflake.check('print("∑")'))
 
+        self.assertTrue(autoflake.check('import os  # ∑'))
+
     def test_get_diff_text(self):
         # We ignore the first two lines since it differs on Python 2.6.
         self.assertEqual(
@@ -647,9 +693,9 @@ import my_own_module
 x = 1
 """) as filename:
             output_file = io.StringIO()
-            autoflake.main(argv=['my_fake_program', filename],
-                           standard_out=output_file,
-                           standard_error=None)
+            autoflake._main(argv=['my_fake_program', filename],
+                            standard_out=output_file,
+                            standard_error=None)
             self.assertEqual("""\
 -import re
 -import os
@@ -659,9 +705,9 @@ x = 1
 
     def test_diff_with_nonexistent_file(self):
         output_file = io.StringIO()
-        autoflake.main(argv=['my_fake_program', 'nonexistent_file'],
-                       standard_out=output_file,
-                       standard_error=output_file)
+        autoflake._main(argv=['my_fake_program', 'nonexistent_file'],
+                        standard_out=output_file,
+                        standard_error=output_file)
         self.assertIn('no such file', output_file.getvalue().lower())
 
     def test_diff_with_encoding_declaration(self):
@@ -673,9 +719,9 @@ import my_own_module
 x = 1
 """) as filename:
             output_file = io.StringIO()
-            autoflake.main(argv=['my_fake_program', filename],
-                           standard_out=output_file,
-                           standard_error=None)
+            autoflake._main(argv=['my_fake_program', filename],
+                            standard_out=output_file,
+                            standard_error=None)
             self.assertEqual("""\
  # coding: iso-8859-1
 -import re
@@ -697,9 +743,9 @@ except ImportError:
     import os
 """) as filename:
             output_file = io.StringIO()
-            autoflake.main(argv=['my_fake_program', '--in-place', filename],
-                           standard_out=output_file,
-                           standard_error=None)
+            autoflake._main(argv=['my_fake_program', '--in-place', filename],
+                            standard_out=output_file,
+                            standard_error=None)
             with open(filename) as f:
                 self.assertEqual("""\
 import foo
@@ -717,9 +763,9 @@ except ImportError:
 
         with temporary_file(line) as filename:
             output_file = io.StringIO()
-            autoflake.main(argv=['my_fake_program', '--in-place', filename],
-                           standard_out=output_file,
-                           standard_error=None)
+            autoflake._main(argv=['my_fake_program', '--in-place', filename],
+                            standard_out=output_file,
+                            standard_error=None)
             with open(filename) as f:
                 self.assertEqual(line, f.read())
 
@@ -739,9 +785,9 @@ except ImportError:
     import sys
 """) as filename:
             output_file = io.StringIO()
-            autoflake.main(argv=['my_fake_program', '--in-place', filename],
-                           standard_out=output_file,
-                           standard_error=None)
+            autoflake._main(argv=['my_fake_program', '--in-place', filename],
+                            standard_out=output_file,
+                            standard_error=None)
             with open(filename) as f:
                 self.assertEqual("""\
 import foo
@@ -757,9 +803,9 @@ except ImportError:
     def test_with_missing_file(self):
         output_file = io.StringIO()
         ignore = StubFile()
-        autoflake.main(argv=['my_fake_program', '--in-place', '.fake'],
-                       standard_out=output_file,
-                       standard_error=ignore)
+        autoflake._main(argv=['my_fake_program', '--in-place', '.fake'],
+                        standard_out=output_file,
+                        standard_error=ignore)
         self.assertFalse(output_file.getvalue())
 
     def test_ignore_hidden_directories(self):
@@ -773,21 +819,21 @@ import os
 """, directory=inner_directory):
 
                     output_file = io.StringIO()
-                    autoflake.main(argv=['my_fake_program',
-                                         '--recursive',
-                                         directory],
-                                   standard_out=output_file,
-                                   standard_error=None)
+                    autoflake._main(argv=['my_fake_program',
+                                          '--recursive',
+                                          directory],
+                                    standard_out=output_file,
+                                    standard_error=None)
                     self.assertEqual(
                         '',
                         output_file.getvalue().strip())
 
     def test_redundant_options(self):
         output_file = io.StringIO()
-        autoflake.main(argv=['my_fake_program',
-                             '--remove-all', '--imports=blah', __file__],
-                       standard_out=output_file,
-                       standard_error=output_file)
+        autoflake._main(argv=['my_fake_program',
+                              '--remove-all', '--imports=blah', __file__],
+                        standard_out=output_file,
+                        standard_error=output_file)
 
         self.assertIn('redundant', output_file.getvalue())
 
@@ -798,9 +844,8 @@ import re, os
 x = os.sep
 print(x)
 """) as filename:
-            process = subprocess.Popen([sys.executable,
-                                        './autoflake',
-                                        '--imports=fake_foo,fake_bar',
+            process = subprocess.Popen(AUTOFLAKE_COMMAND +
+                                       ['--imports=fake_foo,fake_bar',
                                         filename],
                                        stdout=subprocess.PIPE)
             self.assertEqual("""\
@@ -811,7 +856,7 @@ print(x)
 +import os
  x = os.sep
  print(x)
-""", '\n'.join(process.communicate()[0].decode('utf-8').split('\n')[3:]))
+""", '\n'.join(process.communicate()[0].decode().split('\n')[3:]))
 
     def test_end_to_end_with_remove_all_unused_imports(self):
         with temporary_file("""\
@@ -820,9 +865,8 @@ import re, os
 x = os.sep
 print(x)
 """) as filename:
-            process = subprocess.Popen([sys.executable,
-                                        './autoflake',
-                                        '--remove-all',
+            process = subprocess.Popen(AUTOFLAKE_COMMAND +
+                                       ['--remove-all',
                                         filename],
                                        stdout=subprocess.PIPE)
             self.assertEqual("""\
@@ -831,7 +875,7 @@ print(x)
 +import os
  x = os.sep
  print(x)
-""", '\n'.join(process.communicate()[0].decode('utf-8').split('\n')[3:]))
+""", '\n'.join(process.communicate()[0].decode().split('\n')[3:]))
 
     def test_end_to_end_with_error(self):
         with temporary_file("""\
@@ -840,15 +884,14 @@ import re, os
 x = os.sep
 print(x)
 """) as filename:
-            process = subprocess.Popen([sys.executable,
-                                        './autoflake',
-                                        '--imports=fake_foo,fake_bar',
+            process = subprocess.Popen(AUTOFLAKE_COMMAND +
+                                       ['--imports=fake_foo,fake_bar',
                                         '--remove-all',
                                         filename],
                                        stderr=subprocess.PIPE)
             self.assertIn(
                 'redundant',
-                process.communicate()[1].decode('utf-8'))
+                process.communicate()[1].decode())
 
 
 @contextlib.contextmanager
@@ -857,7 +900,7 @@ def temporary_file(contents, directory='.', prefix=''):
     f = tempfile.NamedTemporaryFile(suffix='.py', prefix=prefix,
                                     delete=False, dir=directory)
     try:
-        f.write(contents.encode('utf8'))
+        f.write(contents.encode())
         f.close()
         yield f.name
     finally:
